package com.zskk.controller;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.List;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.jfinal.kit.PropKit;
import com.jfinal.weixin.sdk.api.AccessTokenApi;
import com.jfinal.weixin.sdk.api.ApiConfigKit;
import com.jfinal.weixin.sdk.api.ApiResult;
import com.jfinal.weixin.sdk.api.CallbackIpApi;
import com.jfinal.weixin.sdk.api.CustomServiceApi;
import com.jfinal.weixin.sdk.api.MenuApi;
import com.jfinal.weixin.sdk.api.QrcodeApi;
import com.jfinal.weixin.sdk.api.ShorturlApi;
import com.jfinal.weixin.sdk.api.UserApi;
import com.jfinal.weixin.sdk.jfinal.ApiController;
import com.zskk.model.Exams;
import com.zskk.model.Institution;
import com.zskk.model.Report;
import com.zskk.model.UserWechat;
import com.zskk.service.ServiceFactory;
import com.zskk.service.WeixinService;
import com.zskk.service.bean.WxJSConfigBean;

public class WeixinApiController extends ApiController {

    /**
	 * 为WeixinConfig onLineTokenUrl处提供AccessToken
	 * 
	 * 此处是为了开发测试和生产环境同时使用一套appId时为开发测试环境提供AccessToken
	 * 
	 * 设计初衷：https://www.oschina.net/question/2702126_2237352
	 */
	public void getToken() {
		try {
			String json = AccessTokenApi.getAccessToken().getAccessToken();
			renderText(json);
		} catch (Exception e) {
			// TODO: handle exception
			renderText(e.toString());
		}
	}

    /**
     * 获取公众号菜单
     */
    public void getMenu() {
        ApiResult apiResult = MenuApi.getMenu();
        if (apiResult.isSucceed())
            renderText(apiResult.getJson());
        else
            renderText(apiResult.getErrorMsg());
    }

    /**
     * 创建菜单
     */
    public void createMenu()
    {
        String str = "{\n" +
                "    \"button\": [\n" +
                "        {\n" +
                "            \"name\": \"个人中心\",\n" +
                "            \"sub_button\": [\n" +
                "        {\n" +
                "\t    \"name\": \"我的检查列表\",\n" +
                "\t    \"url\": \"https://wechat.pacsonline.cn/wx_patient/web/examList?appId=" + ApiConfigKit.getApiConfig().getAppId() +"\",\n" +
                "\t    \"type\": \"view\"\n" +
                "        },\n" +
                "        {\n" +
                "\t    \"name\": \"添加绑定\",\n" +
                "\t    \"url\": \"https://wechat.pacsonline.cn/wx_patient/web/addBinding?appId=" + ApiConfigKit.getApiConfig().getAppId() +"\",\n" +
                "\t    \"type\": \"view\"\n" +
                "        },\n" +
                "        {\n" +
                "\t    \"name\": \"远程诊断\",\n" +
                "\t    \"url\": \"https://wechat.pacsonline.cn/wx_patient/web/remoteDiagnosis?appId=" + ApiConfigKit.getApiConfig().getAppId() +"\",\n" +
                "\t    \"type\": \"view\"\n" +
                "        }\n" +
                "    ]\n" +
                "        },\n" +
                "        {\n" +
                "            \"name\": \"增值服务\",\n" +
                "            \"sub_button\": [\n" +
                "        {\n" +
                "\t    \"name\": \"大病筹款\",\n" +
                "\t    \"url\": \"https://www.shuidichou.com/luban/f45t2je2sard/1?channel=ad_search_sem&source=app_zkwx\",\n" +
                "\t    \"type\": \"view\"\n" +
                "        },\n" +
                "        {\n" +
                "\t    \"name\": \"健易保\",\n" +
                "\t    \"url\": \"http://leo.spread.newtank.cn/group0930\",\n" +
                "\t    \"type\": \"view\"\n" +
                "        }\n" +
                "    ]\n" +
                "        },\n" +
                "        {\n" +
                "      \"name\": \"健康之家\",\n" +
                "\t    \"url\": \"https://mp.weixin.qq.com/s/EUK981Irt7hOb6gOs4JC1Q\",\n" +
                "\t    \"type\": \"view\"\n" +
                "        }\n" +
                "    ]\n" + 
                "}";
        ApiResult apiResult = MenuApi.createMenu(str);
        if (apiResult.isSucceed())
            renderText(apiResult.getJson());
        else
            renderText(apiResult.getErrorMsg());
    }

    /**
     * 获取公众号关注用户
     */
    public void getFollowers()
    {
        ApiResult apiResult = UserApi.getFollows();
        JSONObject json = JSON.parseObject(apiResult.getJson());
        JSONObject json2 = json.getObject("data", JSONObject.class);
        JSONArray jsonArray = json2.getJSONArray("openid");
        List<UserWechat> users = UserWechat.dao.find("SELECT * FROM user_wechat where source='测试微信'");
        for (UserWechat user : users) {
			if (jsonArray.contains(user.getWxOpenid())) {
				user.setSource("辽宁省电子胶片平台");
//				user.update();
			}
		}
        renderText(jsonArray.toJSONString());
    }

    /**
     * 获取用户信息
     */
    public void getUserInfo()
    {
        ApiResult apiResult = UserApi.getUserInfo("ohbweuNYB_heu_buiBWZtwgi4xzU");
        renderText(apiResult.getJson());
    }

    /**
     * 发送模板消息。参数pushType推送类型、openid、source用户所属微信公众号、content消息推送json字符串
     */
    public void sendMsg()
    {
        Integer pushType = getParaToInt("pushType");
        String openid = getPara("openid");
        String source = "中世康恺电子胶片平台";
        String content = getPara("content");
		String appId;
		String source_utf8 = null;
		try {
			source_utf8 = URLDecoder.decode(source, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        switch (source_utf8) {
		case "中世康恺电子胶片平台":
			appId = PropKit.get("appId_zskk");
			break;

		case "黑龙江省电子胶片平台":
			appId = PropKit.get("appId_hlj");
			break;

		case "辽宁省电子胶片平台":
			appId = PropKit.get("appId_ln");
			break;
			
		case "河南省电子胶片平台":
			appId = PropKit.get("appId_hn");
			break;

		default:
			appId = PropKit.get("appId_zskk");
			break;
		}
        
        ApiConfigKit.setThreadLocalAppId(appId);
		WeixinService wService = ServiceFactory.getService(WeixinService.class);
		ApiResult apiResult = wService.sendTemplateMsg(openid, content, pushType, source_utf8);
		renderJson(apiResult);
    }
    
    /**
     * 统一获取微信二维码
     */
    public void unifyGetWxQrcode()
    {
        String reportId = getPara("reportId");
		Report reports = Report.dao.findById(reportId);
		Exams exams = Exams.dao.findById(reports.getExamId());
		Institution institution = Institution.dao.findById(exams.getInstitutionId());
		String appId;
		switch (institution.getWxSource()) {
		case "中世康恺电子胶片平台":
			appId = PropKit.get("appId_zskk");
			break;

		case "黑龙江省电子胶片平台":
			appId = PropKit.get("appId_hlj");
			break;

		case "辽宁省电子胶片平台":
			appId = PropKit.get("appId_ln");
			break;
			
		case "河南省电子胶片平台":
			appId = PropKit.get("appId_hn");
			break;

		default:
			appId = PropKit.get("appId_zskk");
			break;
		}
//		redirect("http://wechat.pacsonline.cn/wx_patient/api/getQrcode?appId=" + appId + "&reportId=" + reportId);
        ApiConfigKit.setThreadLocalAppId(appId);
        getQrcode(reportId);
    }
    
    /**
     * 获取参数二维码
     */
    public void getQrcode(String reportId)
    {
		Report reports = Report.dao.findById(reportId);
		if (reportId == null || reports == null) {
			renderNull();
			return;
		}
		if (reports.getQrCode() == null || reports.getQrCode().isEmpty()) {		
			String str = "{\"expire_seconds\": 2592000, \"action_name\": \"QR_STR_SCENE\", \"action_info\": {\"scene\": {\"scene_str\": \"ExamQR_" + reports.getExamId() + "\"}}}";
	        ApiResult apiResult = QrcodeApi.create(str);
	        apiResult.getStr("url");
			reports.setQrCode(apiResult.getStr("url"));
			reports.update();
	        renderText(apiResult.getJson());
		}else {
			//ApiResult qrresult = new ApiResult(reports.getQrCode());
			renderText(reports.getQrCode());;
		}
    }
    
    /*/
     * 获取JS-SDK的配置信息
     */
    public void getJsSdkConfig() 
    {
    	String origin = this.getRequest().getHeader("Origin");
		this.getResponse().setHeader("Access-Control-Allow-Origin", origin);
		this.getResponse().setHeader("Access-Control-Allow-Credentials", Boolean.TRUE.toString());
    	WeixinService wService = ServiceFactory.getService(WeixinService.class);
    	WxJSConfigBean wxJSConfigBean = wService.getWeixinJSSDKConfig(this.getRequest().getHeader("Referer"));
    	renderJson(wxJSConfigBean);
	}
	
    /**
     * 长链接转成短链接
     */
    public void getShorturl()
    {
        String longUrl = getPara("longUrl");
        String str = "{\"action\":\"long2short\"," +
                "\"long_url\":\""+longUrl +"\"}";
        ApiResult apiResult = ShorturlApi.getShorturl(str);
        renderText(apiResult.getJson());
    }

    /**
     * 获取客服聊天记录
     */
    public void getRecord()
    {
        String str = "{\n" +
                "    \"endtime\" : 987654321,\n" +
                "    \"pageindex\" : 1,\n" +
                "    \"pagesize\" : 10,\n" +
                "    \"starttime\" : 123456789\n" +
                " }";
        ApiResult apiResult = CustomServiceApi.getRecord(str);
        renderText(apiResult.getJson());
    }

    /**
     * 获取微信服务器IP地址
     */
    public void getCallbackIp()
    {
        ApiResult apiResult = CallbackIpApi.getCallbackIp();
        renderText(apiResult.getJson());
    }
    
}

